@@ -3,6 +3,7 @@ import re
 from unicodedata import category
 
 # System library imports
+from qtpy import QT6
 from qtpy import QtCore, QtGui, QtWidgets
 
 
@@ -19,7 +20,9 @@ class CallTipWidget(QtWidgets.QLabel):
             text edit widget.
         """
         assert isinstance(text_edit, (QtWidgets.QTextEdit, QtWidgets.QPlainTextEdit))
-        super(CallTipWidget, self).__init__(None, QtCore.Qt.ToolTip)
+        super().__init__(None, QtCore.Qt.ToolTip)
+
+        text_edit.destroyed.connect(self.deleteLater)
 
         self._hide_timer = QtCore.QBasicTimer()
         self._text_edit = text_edit
@@ -62,7 +65,7 @@ class CallTipWidget(QtWidgets.QLabel):
             elif etype == QtCore.QEvent.Leave:
                 self._leave_event_hide()
 
-        return super(CallTipWidget, self).eventFilter(obj, event)
+        return super().eventFilter(obj, event)
 
     def timerEvent(self, event):
         """ Reimplemented to hide the widget when the hide timer fires.
@@ -78,21 +81,25 @@ class CallTipWidget(QtWidgets.QLabel):
     def enterEvent(self, event):
         """ Reimplemented to cancel the hide timer.
         """
-        super(CallTipWidget, self).enterEvent(event)
+        super().enterEvent(event)
         self._hide_timer.stop()
 
     def hideEvent(self, event):
         """ Reimplemented to disconnect signal handlers and event filter.
         """
-        super(CallTipWidget, self).hideEvent(event)
-        self._text_edit.cursorPositionChanged.disconnect(
-            self._cursor_position_changed)
+        super().hideEvent(event)
+        # This fixes issue jupyter/qtconsole#383
+        try:
+            self._text_edit.cursorPositionChanged.disconnect(
+                self._cursor_position_changed)
+        except TypeError:
+            pass
         self._text_edit.removeEventFilter(self)
 
     def leaveEvent(self, event):
         """ Reimplemented to start the hide timer.
         """
-        super(CallTipWidget, self).leaveEvent(event)
+        super().leaveEvent(event)
         self._leave_event_hide()
 
     def paintEvent(self, event):
@@ -104,21 +111,31 @@ class CallTipWidget(QtWidgets.QLabel):
         painter.drawPrimitive(QtWidgets.QStyle.PE_PanelTipLabel, option)
         painter.end()
 
-        super(CallTipWidget, self).paintEvent(event)
+        super().paintEvent(event)
 
     def setFont(self, font):
         """ Reimplemented to allow use of this method as a slot.
         """
-        super(CallTipWidget, self).setFont(font)
+        super().setFont(font)
 
     def showEvent(self, event):
         """ Reimplemented to connect signal handlers and event filter.
         """
-        super(CallTipWidget, self).showEvent(event)
+        super().showEvent(event)
         self._text_edit.cursorPositionChanged.connect(
             self._cursor_position_changed)
         self._text_edit.installEventFilter(self)
 
+    def deleteLater(self):
+        """ Avoids an error when the widget has already been deleted.
+
+            Fixes jupyter/qtconsole#507.
+        """
+        try:
+            return super().deleteLater()
+        except RuntimeError:
+            pass
+
     #--------------------------------------------------------------------------
     # 'CallTipWidget' interface
     #--------------------------------------------------------------------------
@@ -153,9 +170,12 @@ class CallTipWidget(QtWidgets.QLabel):
         # Locate and show the widget. Place the tip below the current line
         # unless it would be off the screen.  In that case, decide the best
         # location based trying to minimize the  area that goes off-screen.
-        padding = 3 # Distance in pixels between cursor bounds and tip box.
+        padding = 3  # Distance in pixels between cursor bounds and tip box.
         cursor_rect = text_edit.cursorRect(cursor)
-        screen_rect = QtWidgets.qApp.desktop().screenGeometry(text_edit)
+        if QT6:
+            screen_rect = text_edit.screen().geometry()
+        else:
+            screen_rect = QtWidgets.QApplication.instance().desktop().screenGeometry(text_edit)
         point = text_edit.mapToGlobal(cursor_rect.bottomRight())
         point.setY(point.y() + padding)
         tip_height = self.size().height()
@@ -240,7 +260,7 @@ class CallTipWidget(QtWidgets.QLabel):
             # If Enter events always came after Leave events, we wouldn't need
             # this check. But on Mac OS, it sometimes happens the other way
             # around when the tooltip is created.
-            QtWidgets.qApp.topLevelAt(QtGui.QCursor.pos()) != self):
+            QtWidgets.QApplication.instance().topLevelAt(QtGui.QCursor.pos()) != self):
             self._hide_timer.start(300, self)
 
     def _format_tooltip(self, doc):
